.batchPca <- function(object, map, verbose=interactive()        ,...) {
  
  if(verbose)
    cat("Calculating batch pca models...\n")

  res <- lapply(map,
                function(y) {
                  pca(object, subset=y, ...)
                })

  if(!is.null(names(map)))
    names(res) <- names(map)
  else
    names(res) <- as.character(1:length(map))

  if(verbose)
    cat("Done\n")

  new("batchPcaRes", pcaModels=res)
}



setMethod("batchPca", "exprSet",
           function(object, map, verbose=TRUE,...) {
             object2 <- t(exprs(object))
             colnames(object2) <- geneNames(object)
             rownames(object2) <- sampleNames(object)
             if(is.null(colnames(object2)))
               colnames(object2) <- as.character(1:ncol(object2))
             if(is.null(rownames(object2)))
               rownames(object2) <- as.character(1:nrow(object2))
             .batchPca(object2, map, verbose, ...)
           })

setMethod("batchPca", "matrix",
          function(object, map, verbose=TRUE,...) {
            object2 <- t(object)
            if(is.null(colnames(object2)))
               colnames(object2) <- as.character(1:ncol(object2))
             if(is.null(rownames(object2)))
               rownames(object2) <- as.character(1:nrow(object2))
            .batchPca(object2, map, verbose, ...)
          })

setMethod("print", "batchPcaRes",
          function(x,...) {
            cat(length(x@pcaModels), " PCA models\n")
            cat("First model:\n")
            print(x@pcaModels[[1]])
          })
          
setMethod("slplot", "batchPcaRes",
          function(object, pcs=c(1,2), scoresLoadings=c(TRUE, TRUE),
                   sl="def", ll="def", hotelling=0.95, rug=TRUE,which=1,...) {
            if(length(which) > 1)
              stop("only defined for one model at a time")
            tmp <- object[[which]]

            if(sl == "def")
              sl <- rownames(tmp@scores)
            if(ll == "def")
              ll <- rownames(tmp@loadings)
            slplot(tmp, pcs=pcs, scoresLoadings=scoresLoadings, sl=sl,
                   ll=ll, hotelling=hotelling, rug=rug,...)
          })

setMethod("summary", "batchPcaRes",
          function(object) {
            varSizes <- unlist(lapply(object@pcaModels, function(x) x@nvar))
            samSizes <- unlist(lapply(object@pcaModels, function(x) x@nobs))
            meth <- unlist(lapply(object@pcaModels, function(x) x@method))
            scal <- unlist(lapply(object@pcaModels, function(x) x@scaled))
            cent <- unlist(lapply(object@pcaModels, function(x) x@centered))
            cat(length(object@pcaModels), " PCA models\n")
            cat("Used method:\t", unique(meth), "\n")
            cat("Scaling:\t", unique(scal),"\n")
            cat("Centering:\t", unique(cent), "\n")
            if(length(unique(varSizes)) > 5)
              cat("n Variables:\t", unique(varSizes)[1:5],"...\n")
            else
              cat("n Variables:\t", unique(varSizes), "\n")
            if(length(unique(samSizes)) > 5)
              cat("n Samples:\t", unique(samSizes)[1:5],"...\n")
            else
              cat("n Samples:\t", unique(samSizes), "\n")
            invisible(list(varSizes=varSizes, samSizes=samSizes))
          })

setMethod("[", "batchPcaRes",
          function(x, i, j=NULL, drop=FALSE) {
            if(length(i) == 1)
              return(x@pcaModels[[i]])
            new("batchPcaRes", pcaModels=x@pcaModels[i])
          })

setMethod("length", "batchPcaRes",
          function(x) {
            length(x@pcaModels)
          })
