scoreModels <- function(x, scoretype=c("mclust", "silhouette", "weightedsilhouette"),
                        verbose=interactive(),...) {
  
  scoretype=match.arg(scoretype)

  tmp <- x[1]

  clusterings <- matrix(NA, nrow=tmp@nobs, ncol=length(x))
  certainty <- matrix(NA, nrow=tmp@nobs, ncol=length(x))
  scores <- rep(NA, length(x))
  sizes <- rep(NA, length(x))
  rn <- names(x@pcaModels)
  
  if(verbose)
    cat("Evaluating", length(x), "models\n")
  
  for(i in 1:length(x)) {
    if(verbose)
      progBar(i, length(x))
    cl <- clusterGeneClass(x[i], scoretype=scoretype,...)
    scores[i] <- cl$score
    clusterings[,i] <- cl$clustering
    certainty[,i] <- cl$certainty
    sizes[i] <- x[i]@nvar
  }
  names(scores) <- rn
  colnames(clusterings) <- rn
  colnames(certainty) <- rn
  names(sizes) <- rn

  res <- new("modelScores")
  res@scores=scores
  res@clusterings=clusterings
  res@sizes=sizes
  res@certainty=certainty
  res@scoretype=scoretype
  
  if(verbose)
    cat("Done\n")
  
  res
}

setMethod("[", "modelScores",
          function(x, i, j=NULL, drop=FALSE, ...) {
            new("modelScores", scores=x@scores[i], clusterings=x@clusterings[,i, drop=FALSE],
                sizes=x@sizes[i], certainty=x@certainty[,i, drop=FALSE], scoretype=x@scoretype)
          })

setMethod("names", "modelScores",
          function(x) {
            names(x@scores)
          })
