capiu <- function(x, map, labels=NULL, ...) {

  if(inherits(x, "exprSet"))
    x <- exprs(x)

  if(is.null(colnames(x)))
    colnames(x) <- as.character(1:ncol(x))

  if(is.null(rownames(x)))
    rownames(x) <- as.character(1:nrow(x))

  models <- batchPca(x, map, ...)
  scores <- scoreModels(models, ...)

  nodes <- new("udt")
  if(is.null(labels))
    nodes@labels <- colnames(x)
  else
    nodes@labels <- as.character(labels)
  names(nodes@labels) <- colnames(x)
  
  nodes <- addNodes(nodes, nodeGen(scores, models, map, eset=x,...))

  if(getType(nodes, 1) == "leaf")
    stop("no significant splits found")
  
  ## iterate on all the children of the first node until no
  ## leaf has more members than two
  continue <- TRUE
  i <- 1
  while(continue) {
    ## constructs all the nodes of the childrens of node i
    for(j in 1:getChildren(nodes, i)) {
      members <- getMembers(nodes, i, j, translate=FALSE)
      members <- match(members, colnames(x))
      tempX <- x[, members]
        
      models <- batchPca(tempX, map, ...)
      scores <- scoreModels(models, ...)
      temp <- nodeGen(scores, models, map, eset=tempX,...)
      nodes <- addNodes(nodes, temp)
    }
    ## check for end of iteration (if any of the produced nodes has
    ## children > 0 (i.e. the node is not a node but a leaf)
    children <- NULL
    for(k in (i+1):length(nodes))
      children <- c(children, getChildren(nodes, k))
    
    if(all(children == 0)) 
      continue <- FALSE
    ## figure what the next node to process is, i.e. next node which
    ## has childrens > 0
    if(continue) {
      i <- i + 1
      while(getChildren(nodes, i) == 0) 
        i <- i + 1
    }
  }
  nodes
}

toDot <- function(x, filename, graphname="G", goTerm=TRUE, goId=TRUE,
                  compile=TRUE, useLabels=TRUE) {

  ## what...creates a "dot" file of a nodes object in order to generate a 
  ## graphical tree on the data                                           
  ## x...a nodes object                                                   
  ## file..output file                                                    
  ## graphname...the name of the graph                                    
  ## goTerm...indicates whether the corresponding GO Term should be added 
  ## goId...indicates whether the corresponding GO Ids should be added    
  ## when...2005-02-28                                                    
  ## who...redestig@mpimp-golm.mpg.de                                     

  file <- file(filename, "w")

  cat(file=file, "digraph", graphname, "{\n")
                                        #cat(file=file, "size=\"10,7\"\n", append=T, sep="")
  for(i in 1:length(x)) {

    ## write all nodes
    if(getType(x, i) == "node") {
      cat(file=file, i, "[shape=record, fontsize=10, label=\"{", append=TRUE, sep="")
      
      ## write the ranks column
      for(j in 1:(length(getRanks(x, i)))) {
        if(i == 1 && j == 1) cat(file=file, "Rank|", append=TRUE, sep="")
        cat(file=file, getRanks(x, i)[j], append=TRUE, sep="")
        if (j < length(getRanks(x, i))) {
          cat(file=file, "|", append=TRUE, sep="")
        } else {
          cat(file=file, "}|{", append=TRUE, sep="")
        }        

      }

      ## write class name/id column
      for(j in 1:length(getGeneClasses(x, i))) {
        if(i == 1 && j == 1) cat(file=file, "GO Term\\nGO ID|", append=TRUE, sep="")
        if(goId) cat(file=file, getGeneClasses(x, i)[j], append=TRUE, sep="")
        if(goId && goTerm) cat(file=file, "\\n", append=TRUE, sep="")
        if(goTerm) {
          if(exists(getGeneClasses(x, i)[j], GOTERM)) {
            name <- NULL
            name <- unlist(try(getGOTerm(getGeneClasses(x, i)[j]), TRUE))
            if (is.null(name)) {
              name <- "???"
            }
            else if(nchar(name) > 33) {
              name <- paste(substr(name, 1, 31), "..", sep="")
            }
          }
          else {
            name <- getGeneClasses(x, i)[j]
          }
          cat(file=file,name, append=TRUE, sep="")
        }
        if(j != length(getGeneClasses(x, i))) cat(file=file, "|", append=TRUE, sep="")
      }
      cat(file=file, "}|{", append=TRUE, sep="")

      ## write scores column
      for(j in 1:(length(getScores(x, i)))) {
        if(j == 0) break
        temp <- getScores(x, i)
        if(i == 1 && j == 1) cat(file=file, "S|", append=TRUE, sep="")
        cat(file=file, sprintf("%.2f", temp[j]), append=TRUE, sep="")
        if (j < length(temp)) {
          cat(file=file, "|", append=TRUE, sep="")
        } else {
          cat(file=file, "}|{", append=TRUE, sep="")
        }        
      }

      ## write the size column
      for(j in 1:(length(getRanks(x, i)))) {
        if(i == 1 && j == 1) cat(file=file, "Size|", append=TRUE, sep="")
        cat(file=file, getSizes(x, i)[j], append=TRUE, sep="")
        if (j < length(getSizes(x, i))) {
          cat(file=file, "|", append=TRUE, sep="")
        } else {
          cat(file=file, "}", append=TRUE, sep="")
        }
      }

      cat(file=file, "\"];\n", append=TRUE, sep="")
    }

    ## write leaves
    if(getType(x, i) == "leaf") {
      if(useLabels)
        members <- unlist(getMembers(x, i))
      else
        members <- unlist(getMembers(x, i, translate=FALSE))
      if(length(members) / 2 != floor(length(members) / 2)) {
        members <- c(members, "")
      }
      members <- matrix(members, ncol=2)
      cat(file=file, i, "[label=\"", append=TRUE)
      for(q in 1:nrow(members)) {
        cat(file=file, members[q,], append=TRUE)
        if(members[q,2] != "" && q != nrow(members)) cat(file=file, "\\n", append=TRUE)
      }
      cat(file=file, "\"];\n", append=TRUE)
    }
  }

  ## write the edges
  max <- 1
  for(i in 1:length(x)) {
    if(getChildren(x, i) != 0) {
      k <- 1
      for(j in (max + 1):(max + getChildren(x, i))) {
        cat(file=file, i, "->", j, "[label=\"\"];\n", append=TRUE)
        k <- k + 1
      }
      max <- max + getChildren(x, i)
    }
  }

  cat(file=file, "}\n", append=TRUE, sep="")

  if(compile) {
    cat(sprintf("compiling %s\n", filename))
    system(paste("dot -Tpng", filename, ">", sub(".dot", ".png", filename)))
  }

  close(file)
}



setMethod("getSizes", "udt",
          function(object, i) {
            tmp <- object@nodes[[i]]
            tmp@sizes
          })

setMethod("getScores", "udt",
          function(object, i) {
            tmp <- object@nodes[[i]]
            tmp@scores
          })

setMethod("getGeneClasses", "udt",
          function(object, i) {
            tmp <- object@nodes[[i]]
            tmp@classes
          })

setMethod("getRanks", "udt",
          function(object, i) {
            tmp <- object@nodes[[i]]
            tmp@ranks
          })

setMethod("getChildren", "udt",
          function(object, i) {
            tmp <- object@nodes[[i]]
            tmp@children
          })

setMethod("getMembers", "udt",
          function(object, i, j=NULL, translate=TRUE) {
            tmp <- object@nodes[[i]]
            if(!is.null(j)) {
              members <- tmp@members[[j]]
              if(translate && !is.null(object@labels))
                return(object@labels[members])
              else
                return(members)
            }
            else {
              members <- tmp@members
              if(translate && !is.null(object@labels))
                return(lapply(members, function(x) object@labels[x]))
              else
                return(members)
            }
          })

setMethod("getType", "udt",
          function(object, i) {
            tmp <- object@nodes[[i]]
            tmp@type
          })

setMethod("addNodes", "udt",
          function(object, node) {
            if(!is.null(object@nodes)) {
              object@nodes[[length(object@nodes) + 1]] <- node
              daNodes <- object@nodes
            }
            else
              daNodes <- list(node)
            res <- new("udt")
            res@nodes <- daNodes
            res@labels <- object@labels
            res
          })
setMethod("length", "udt",
          function(x) {
            length(x@nodes)
          })
