\name{clusterGeneClass}
\alias{clusterGeneClass}
\alias{clusterGeneClass,matrix-method}
\alias{clusterGeneClass,pcaRes-method}
\title{Cluster the samples using information from a gene class.}
\description{This function provides functionality for clustering the
  samples microarray experiment using information from one gene class
  only. The gene class should be pre-processed with PCA.}
\usage{clusterGeneClass(object, clNum=2, scoretype=c("mclust", "weightedsilhouette", "silhouette"),...) }
\arguments{
  \item{object}{A matrix containing raw gene expression measurements
    or a pcaRes object.}
  \item{clNum}{The amount of clusters to search for. More than two is experimental.}
  \item{scoretype}{The scoretype to use:
    \describe{
      \item{mclust}{Use mixture model clustering to fit Gaussian mixture
	model. Score is the log-likelihood ratio between uni-modal and
	clNum-model clusterings}
      \item{silhouette}{Clustering is done by PAM from the
	cluster package and the score is the average silhouette width
	from the clusters.}
      \item{weightedsilhouette}{Same as silhouette but final score is
	weighted by the entropy of the cluster sizes to give preference
	for similar sized clusters}
  }}
  \item{...}{Pass through arguments}
}
\value{
  A list with clustering, certainty, score, clNum and size of the geneclass
}
\author{Henning Redestig}
\seealso{pca}
\examples{
data(golubMergeSub)
myRandomMapping <- sapply(1:10, function(x) sample(1:1000, sample(5:200)))
models <- batchPca(golubMergeSub, myRandomMapping)
score <- clusterGeneClass(models[1], 2, "mclust")
}
\keyword{tree}
