\name{scoreModels}
\alias{scoreModels}
\title{Compute scores for the computed models in a batchPca object}
\description{This method computes the clusterings and scores of choice
  for each element (model) in a \code{bactPca} object. These scores can then
  be used to find the best model according to the measure of choice. In
  an unsupervised decision tree setting this model is used for splitting
  the samples.}
\usage{scoreModels(x, scoretype=c("mclust", "silhouette",
"weightedsilhouette"), verbose=interactive(), ...)}
\arguments{
  \item{x}{A batchPca object}
  \item{scoretype}{Which score to use; implemented scores are defined as
    such:
    \describe{
      \item{mclust}{Fit uni-modal and bi-modal distributions
	to a uni-variate distribution using the mclust package and
	compute score by log-likelihood between the two obtained models}
      \item{silhouette}{PAM is used to cluster data and average
	Silhouette Index of all clusters is the total score.}
      \item{weightedsilhouette}{Same as silhouette but multiplied by the relative.
	entropy of the cluster sizes to give small preference for equal
	sized clusters.}
    }
  }
  \item{verbose}{If TRUE a message is printed a start and end of
    calulation}
  \item{...}{Only used for passing through arguments}
}
\details{The clusterings are done with \code{pam} from the package
  \pkg{cluster} except when \bold{modelSelect} is used in which case the
  EM-algorithm \code{EMclust} from the \code{mclust} package is used.}
\value{A \code{modelScores} object}
\author{Henning Redestig}
\seealso{\code{silhouette}, \code{pam}, \code{batchPca}, \code{pca}}
\examples{
data(golubMergeSub)
myRandomMapping <- sapply(1:10, function(x) sample(1:1000, sample(5:200)))
models <- batchPca(golubMergeSub, myRandomMapping)
scores <- scoreModels(models)
}
\keyword{tree}
